//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, Sept-2010
//-----------------------------------------------------------------------------
//
// TOPconfigure.cc
// define TOP counter configuration
// note: PMT should be defined prior to Qbars
//-----------------------------------------------------------------------------
//*****************************************************************************

#include "TOPconfig.h"
#include <iostream>
#include <math.h>

using namespace std;

void TOPconfigure(int geo, int PMTchoice, int QEchoice, double Redge)
{

  cout << endl << "Selecting TOP geometry configuration #" << geo << 
    endl << endl;

  TOPvolume(116.5, 125.0, -85.0, 192.0);

  setBfield(1.5);

  switch (PMTchoice)
    {
    case 1:
      setPMT(2.75, 2.75, 2.20, 2.20, 4, 4); // SL-10, 4x4
      break;
    case 2:
      setPMT(5.90, 5.90, 5.20, 5.20, 8, 8); // Photonis XP85112
      break;
    default:
      cout << "unknown PMTchoice -> default taken";
      setPMT(2.75, 2.75, 2.20, 2.20, 4, 4); // SL-10, 4x4
    }

  double frac[3]={0.5815, 0.2870, 0.1315};
  double mean[3]={-13.59e-3, 29.03e-3, 273.0e-3};
  double sigma[3]={31.97e-3, 53.39e-3, 340.2e-3};
  setTTS(3, frac, mean, sigma);
  switch (QEchoice)
    {
    case 1:
      setQE("data/qe_GaAsP400nm.dat", 0.35);	
      break;
    case 2:
      setQE("data/qe_multialkali350nm.dat", 0.60);	
      break;
    case 3:
      setQE("data/qe_supbialkali350nm.dat", 0.60);	
      break;
    case 4:
      setQE("data/qe_supbialkali400nm.dat", 0.60);	
      break;
    case 5:
      setQE("data/qe_supbialkali450nm.dat", 0.60);	
      break;
    default:
      cout << "unknown QEchoice -> default taken";
      setQE("data/qe_GaAsP400nm.dat", 0.35);	
    }
  setTDC(10, 50.E-3);

  double Pi=4.*atan(1.);
  
  int n=16;             // number of bars in phi
  double Dphi=2*Pi/n;
  double Phi=0;

  int id;
  double R=120.0;       // innner bar surface radius
  double A=45.4;        // bar width
  double B=2.0;         // bar thickness
  double z1, z2;        // backward, forward bar position
  double DzExp=10.0;    // expansion volume length
  double YsizExp=5.05;  // expansion volume height
  double YsizPMT=2*2.8; // height to arrange 2 rows of PMT

  //// for 2-bar option only ////
  double zMid=107.0;
  double DzGap=1.0;
  ///////////////////////////////

  setEdgeRoughness(Redge);
  switch (geo)
    {	
    case 1: //two-readout f-TOP with spherical mirror (2-bar)
      z1=-85.0;
      z2=185.0;
      for(int i=0; i<n; i++){
	id=setQbar(A, B, z1, zMid, R, 0, Phi, PMT, SphericM);
	setMirrorRadius(id, 500);
	id=setQbar(A, B, zMid+DzGap, z2, R, 0, Phi, None, PMT);
	Phi+=Dphi;
      }
      break;
    case 2: //focusing i-TOP, asymmetric Prism, spherical mirror (1-bar)
      z1=-76.0;
      z2=192.0;
      for(int i=0; i<n; i++){
	id=setQbar(A, B, z1, z2, R, 0, Phi, PMT, SphericM);
	setMirrorRadius(id, 700);
	addExpansionVolume(id, Left, Prism, DzExp, B/2, B/2-YsizExp);
	arrangePMT(id, Left, A, YsizPMT);
	Phi+=Dphi;
      }
      break;
    default:
      for(int i=0; i<n; i++){
	z1=-75.0;
	z2=190.0;
	id=setQbar(A, B, z1, z2, R, 0, Phi, PMT, PMT);
	Phi+=Dphi;
      }
      cout << "Non-existing configuration -> default choosen instead"
	   << endl << endl;
    }		

  TOPfinalize();
}

