!-----------------------------------------------------------------------
! TOP counter for Belle: Digitization
! M. Staric, sept-2008
! units: GeV, GeV/c, cm, ns, cm/ns, T, nm
!-----------------------------------------------------------------------

      SUBROUTINE TOP_DIGI

      IMPLICIT NONE
#include "TOP_PHOT.fi"
#include "TOP_GEO.fi"
#include "TOP_EXT.fi"
#include "TOP_PMT.fi"
#include "TOP_TDC.fi"
#include "TOP_DATA.fi"
#include "TOP_MTRA.fi"
      INTEGER I, K, L, LR, M, ICH, MCHA, NCHA, II
      INTEGER LR_SIDE(0:1)
      INTEGER NGEN, IGEN
      REAL T_BEAM, X, Y, T, YUP, YDN
      REAL GAUS_RND, RND_NUM, PMT_TTS
      INTEGER POISSON_RND, ICH_DIGI
!     time buffer
      integer NSIZ, NDEPTH
      PARAMETER (NSIZ=65536)
      PARAMETER (NDEPTH=16)
      REAL TBUF(NDEPTH,0:NSIZ-1)
      INTEGER NT(0:NSIZ-1)

      MCHA=PMT_NX*PMT_NY*NMA_PMT ! max # of Q-bar channels
      NCHA=MCHA*NUM_MOD         ! max # of all TOP channels
      IF(NCHA.GT.NSIZ) THEN
         PRINT*,'***Error (top_digi): NCHA>NSIZ, NCHA=',NCHA
         PRINT*,'--> Increase parameter value ',
     &           'srcF77/top_digi.F::NSIZ, clean object files, ',
     &           'and recompile!'
         STOP
      ENDIF

!     reset
      N_DATA=0
      DO I=0,NCHA-1
         NT(I)=0
      ENDDO

!     simulate interaction time relative to RF clock
      T_BEAM=GAUS_RND(SIG_BEAM)

!     spatial digitization, simulate PMT time response in respect to RF
      DO I=1,N_PHOT
         PHOT_ICH(I)=-1
         CALL SPATIAL_DIGI(I,ICH,M)
         IF(ICH.LT.0) CYCLE
         T=T_BEAM+PHOT_T(I)+PMT_TTS()
         IF(T.LT.0.OR.T.GT.TIME_RANGE*2) CYCLE
         PHOT_ICH(I)=ICH
         K=ICH+MCHA*M
         CALL PUT_TO_TIMEBUF(T,TBUF(1,K),NT(K),NDEPTH)
         IF(T.LT.TIME_RANGE) THEN
            K=PHOT_MTRA(I)
            MTRA_MPH(K)=MTRA_MPH(K)+1
         ENDIF
      ENDDO

!     add background
      DO M=0,NUM_MOD-1
         LR_SIDE(0)=QBAR_LSIDE(M)
         LR_SIDE(1)=QBAR_RSIDE(M)
         DO L=0,1
            IF(LR_SIDE(L).EQ.0) CYCLE
            LR=L
            IF(LR_SIDE(L).GT.1) LR=1-L
            II=QBAR_EXT(LR,M)
            IF(II.GT.0) THEN
               YUP=EXT_YUP(II)
               YDN=EXT_YDN(II)
            ELSE
               YUP=QBAR_B(M)/2
               YDN=-QBAR_B(M)/2
            ENDIF
            NGEN=POISSON_RND(BGR_PER_QBAR)
            DO IGEN=1,NGEN
               X=QBAR_A(M)*(0.5-RND_NUM())
               Y=YDN+(YUP-YDN)*RND_NUM()
               ICH=ICH_DIGI(X,Y,LR,M)
               IF(ICH.LT.0) CYCLE
               K=ICH+MCHA*M
               T=TIME_RANGE*RND_NUM()
               CALL PUT_TO_TIMEBUF(T,TBUF(1,K),NT(K),NDEPTH)
            ENDDO
         ENDDO
      ENDDO

!     time digitization - simulate CFD and multihit TDC

      DO K=0,NCHA-1
         ICH=MOD(K,MCHA)
         M=K/MCHA
         CALL SIMULATE_CFD(M,ICH,TBUF(1,K),NT(K))
      ENDDO

      END


c---- PMT time spread: MC generator

      REAL FUNCTION PMT_TTS()

      IMPLICIT NONE
#include "TOP_PMT.fi"
      REAL GAUS_RND, RND_NUM
      REAL PROB, S
      INTEGER I

      PROB=RND_NUM()
      S=0.
      DO I=1,TTS_NG
         S=S+TTS_FRAC(I)
         IF(PROB.LT.S) THEN
            PMT_TTS=TTS_T0(I)+GAUS_RND(TTS_SIG(I))
            RETURN
         ENDIF
      ENDDO

      END


c---- position digitization --------------------------------------------

      SUBROUTINE SPATIAL_DIGI(I,ICH,M)

      IMPLICIT NONE
      INTEGER I                 ! photon index [in]
      INTEGER ICH               ! channel ID [out]
      INTEGER M                 ! module ID [out]
#include "TOP_PHOT.fi"
      INTEGER ICH_DIGI
      
      M=PHOT_M(I)
      ICH=ICH_DIGI(PHOT_R(1,I),PHOT_R(2,I),PHOT_LR(I),M)

      END


c---- time digitization --------------------------------------------

      SUBROUTINE PUT_TO_TIMEBUF(T,TIME,NT,NDEPTH)

c--   put T into TIME() in ascending way

      IMPLICIT NONE
      REAL T, TIME(*)
      INTEGER NT, NDEPTH

      INTEGER I, K

      DO I=1,NT
         IF(T.LT.TIME(I)) EXIT
      ENDDO
      IF(I.GT.NDEPTH) RETURN
      NT=MIN(NT+1,NDEPTH)
      DO K=NT,I+1,-1
         TIME(K)=TIME(K-1)
      ENDDO
      TIME(I)=T

      END


      SUBROUTINE SIMULATE_CFD(MDN,ICH,TIME,NT)

      IMPLICIT NONE
      INTEGER MDN               ! module ID
      INTEGER ICH               ! channel ID
      REAL TIME(*)              ! sorted photon times
      INTEGER NT                ! number of photons
#include "TOP_TDC.fi"

      INTEGER I, N
      REAL T, T1

      IF(NT.EQ.0) RETURN

!     simulate pileup and multihit
      T1=TIME(1)
      T=T1
      N=1
      DO I=2,NT
         IF(TIME(I)-T1.LT.CFD_DT) THEN
            T=T+TIME(I)
            N=N+1
         ELSE IF(TIME(I)-TIME(I-1).GT.CFD_DELT) THEN
            T=T/FLOAT(N)
            CALL TIME_DIGI(MDN,ICH,T)
            T1=TIME(I)
            T=T1
            N=1
         ENDIF         
      ENDDO
      T=T/FLOAT(N)
      CALL TIME_DIGI(MDN,ICH,T)

      END


      SUBROUTINE TIME_DIGI(MDN,ICH,T)

      IMPLICIT NONE
      INTEGER MDN               ! module ID
      INTEGER ICH               ! channel ID
      REAL T                    ! time

      INTEGER TDC, TDC_DIGI, K

      TDC=TDC_DIGI(T)
      CALL DATA_PUT(MDN,ICH,TDC,K)

      END


c---- basic digitization functions --------------------------------------

      INTEGER FUNCTION ICH_DIGI(XH,YH,LR,M) ! chID from x,y,lr,m

      IMPLICIT NONE
      REAL XH, YH               ! photon hit coordinates (Q-bar local frame)
      INTEGER LR                ! left/right = 0/1
      INTEGER M                 ! module ID number
#include "TOP_GEO.fi"
#include "TOP_PMT.fi"
      REAL X, Y
      INTEGER IX, IY, IPMTX, IPMTY
      INTEGER ICHAN

      ICH_DIGI=-1
      IF(M.LT.0.OR.M.GE.NUM_MOD) RETURN
      IF(N_PMT(LR,M).EQ.0) RETURN

      X=XH-PMT_X0(LR,M)
      IPMTX=NINT(X/PMT_DX(LR,M))
      IF(IPMTX.LT.0.OR.IPMTX.GE.N_PMTX(LR,M)) RETURN
      X=X-IPMTX*PMT_DX(LR,M)
      IF(ABS(X).GE.PMT_DELX) RETURN
      IX=IFIX((X+PMT_DELX)/PMT_DXCH)

      Y=YH-PMT_Y0(LR,M)
      IPMTY=NINT(Y/PMT_DY(LR,M))
      IF(IPMTY.LT.0.OR.IPMTY.GE.N_PMTY(LR,M)) RETURN
      Y=Y-IPMTY*PMT_DY(LR,M)
      IF(ABS(Y).GE.PMT_DELY) RETURN
      IY=IFIX((Y+PMT_DELY)/PMT_DYCH)

      ICH_DIGI=ICHAN(IX,IY,IPMTX,IPMTY,LR,M)

      END


      INTEGER FUNCTION ICH_DIGIZ(XH,YH,ZH,M) ! chID from x,y,z,m
      
      IMPLICIT NONE
      REAL XH, YH, ZH           ! photon hit coordinates (Q-bar local frame)
      INTEGER M                 ! module ID number
#include "TOP_GEO.fi"
#include "TOP_PMT.fi"
      INTEGER LR, ICH_DIGI

      IF(ZH.LT.0) THEN
         LR=0
      ELSE
         LR=1
      ENDIF
      ICH_DIGIZ=ICH_DIGI(XH,YH,LR,M)
      END


      SUBROUTINE ICHXYZ(ICH,M,XH,YH,ZH) ! inverse of ICH_DIGIZ

      IMPLICIT NONE
      INTEGER ICH, M            ! channel and module ID [in]
      REAL XH, YH, ZH           ! photon hit coordinates (Q-bar local frame)
#include "TOP_GEO.fi"
#include "TOP_PMT.fi"
      INTEGER IX, IY, IPMTX, IPMTY, LR
      REAL X, Y

      XH=0.
      YH=0.
      ZH=0.
      IF(ICH.LT.0) RETURN
      IF(M.LT.0.OR.M.GE.NUM_MOD) RETURN

      CALL ICHIII(ICH,M,IX,IY,IPMTX,IPMTY,LR)
      IF(LR.LT.0) RETURN

      X=(2*IX+1-PMT_NX)*PMT_DXCH/2.
      Y=(2*IY+1-PMT_NY)*PMT_DYCH/2.
      XH=X+IPMTX*PMT_DX(LR,M)+PMT_X0(LR,M)
      YH=Y+IPMTY*PMT_DY(LR,M)+PMT_Y0(LR,M)
      ZH=(LR-0.5)*QBAR_C(M)

      END


      INTEGER FUNCTION ICHAN(IX,IY,IPMTX,IPMTY,LR,M) ! chID from ix,iy,ipmt,lr,m

      IMPLICIT NONE
      INTEGER IX, IY            ! PMT channel number (2D)
      INTEGER IPMTX, IPMTY      ! PMT ID number (2D)
      INTEGER LR                ! exit window ID (0: z>0, 1: z<0)
      INTEGER M                 ! module ID
#include "TOP_GEO.fi"
#include "TOP_PMT.fi"

      ICHAN=IX+PMT_NX*(IY+PMT_NY*(IPMTX+N_PMTX(LR,M)*IPMTY))
      IF(LR.EQ.0) RETURN
      ICHAN=ICHAN+QBAR_NCH0(M)

      END


      SUBROUTINE ICHIII(ICH,M,IX,IY,IPMTX,IPMTY,LR) ! inverse of ICHAN

      IMPLICIT NONE
      INTEGER ICH, M            ! channel and module ID [in]
      INTEGER IX, IY            ! PMT channel number [out]
      INTEGER IPMTX, IPMTY      ! PMT ID number [out]
      INTEGER LR                ! exit window ID (z>0?0:1; ERR=-1) [out]
#include "TOP_GEO.fi"
#include "TOP_PMT.fi"
      INTEGER K

      IX=-1
      IY=-1
      IPMTX=-1
      IPMTY=-1
      LR=-1
      IF(ICH.LT.0) RETURN
      IF(M.LT.0.OR.M.GE.NUM_MOD) RETURN

      IF(ICH.LT.QBAR_NCH0(M)) THEN
         LR=0
         K=ICH
      ELSE
         LR=1
         K=ICH-QBAR_NCH0(M)
      ENDIF
      IF(N_PMT(LR,M).EQ.0) RETURN

      IX=MOD(K,PMT_NX)
      K=K/PMT_NX
      IY=MOD(K,PMT_NY)
      K=K/PMT_NY
      IPMTX=MOD(K,N_PMTX(LR,M))
      K=K/N_PMTX(LR,M)
      IPMTY=MOD(K,N_PMTY(LR,M))
      K=K/N_PMTY(LR,M)
      IF(K.GT.1) LR=-1

      END


      INTEGER FUNCTION TDC_DIGI(T) ! TDC from T

      IMPLICIT NONE
      REAL T                    ! time
#include "TOP_TDC.fi"

      TDC_DIGI=TDC_RANGE
      IF(T.LT.0.OR.T.GT.TIME_RANGE) RETURN
      TDC_DIGI=IFIX(T/TDC_DT)
      END

      
      REAL FUNCTION TDC_TO_T(TDC) ! inverse of TDC_DIGI

      IMPLICIT NONE
      INTEGER TDC               ! TDC time (digitized)
#include "TOP_TDC.fi"
      
      TDC_TO_T=(TDC+0.5)*TDC_DT
      END

