!-----------------------------------------------------------------------
! TOP counter for Belle: Monte Carlo simulation
! M. Staric, sept-2008, nov-2009, feb-2010, sept-2010
! units: GeV, GeV/c, cm, ns, cm/ns, T, nm
!-----------------------------------------------------------------------

      SUBROUTINE TOP_MCGEN

      IMPLICIT NONE
#include "TOP_MTRA.fi"
#include "TOP_PHOT.fi"
#include "TOP_GEO.fi"
      INTEGER ITRA

      IF(.NOT.TOP_INI) THEN
         PRINT*,'*** TOP not initialized ***'
         RETURN
      ENDIF
      N_PHOT=0
      DO ITRA=1,N_MTRA
         CALL CERENKOV_SIMU(ITRA)
      ENDDO
      CALL TOP_DIGI

      END



      SUBROUTINE CERENKOV_SIMU(ITRA) ! with scattering
      
      IMPLICIT NONE
      INTEGER ITRA              ! montecarlo track
#include "TOP_CONS.fi"
#include "TOP_MTRA.fi"
#include "TOP_PHOT.fi"
#include "TOP_GEO.fi"
      REAL E_MIN, E_MAX, E
      REAL P, BETA, COSTHC, RHO
      REAL HLX(10)
      REAL T1, T2, DT, TLEN, T, TOP
      REAL QE, PROB, THC, FIC, GENFRAC, TH0
      REAL RF(3), DIR(3), DIRF(3), POL(3), RF0(3), DIRF0(3)
      INTEGER IGEN, NGEN, MDN, LR, NX(10), NY(10), I
      REAL PHASE_INDEX, GROUP_INDEX, QE_PMT, QE_MAX, RND_NUM
      INTEGER POISSON_RND
      LOGICAL OK

      MTRA_NPH(ITRA)=0
      MTRA_MPH(ITRA)=0
      CALL UNIT_VECT(MTRA_P(1,ITRA),DIR,P)
      IF(P.EQ.0) RETURN
      BETA=P/MTRA_MASS(ITRA)/SQRT(1.+(P/MTRA_MASS(ITRA))**2)
      CALL QE_RANGE(E_MIN,E_MAX)
      COSTHC=1./(BETA*PHASE_INDEX(E_MAX)) ! minimal cos(thc)
      IF(COSTHC.GE.1) RETURN
      GENFRAC=QE_MAX()*(1.-COSTHC**2)
      IF(GENFRAC.LE.0) RETURN

!     track helix
      CALL SET_HELIX(MTRA_R0(1,ITRA),DIR,P,MTRA_CHARGE(ITRA),HLX)

!     crossing of TOP volume
      RHO=SQRT(MTRA_R0(1,ITRA)**2+MTRA_R0(2,ITRA)**2)
      IF(RHO.GE.TOP_R2) RETURN
      CALL PRESEK_HELIX_TOP(HLX,RHO,T1,T2,OK)
      IF(.NOT.OK) RETURN

!     track length inside TOP volumne, number of photons,...
      TLEN=(T2-T1)*C0
      NGEN=POISSON_RND(370.*TLEN*(E_MAX-E_MIN)*GENFRAC)

!     Cerenkov photons
      DT=(T2-T1)/NGEN
      T=T1-DT/2
      TLEN=-DT/2
      DO IGEN=1,NGEN
         T=T+DT
         CALL GET_HLX_RDIR(HLX,T,RF,DIR)
         CALL TO_MODULE(RF,DIR,MDN,OK)
         IF(OK) THEN
            TLEN=TLEN+DT
            E=(E_MAX-E_MIN)*RND_NUM()+E_MIN
            COSTHC=1./(BETA*PHASE_INDEX(E))
            IF(COSTHC.LT.1) THEN
               QE=QE_PMT(E)
               PROB=QE*(1.-COSTHC**2)/GENFRAC
               IF(RND_NUM().LT.PROB) THEN
                  IF(QBAR_RADL.GT.0) THEN
                     TH0=13.6E-3/BETA/P*SQRT(TLEN*C0/QBAR_RADL)
                     CALL MULTIPLE_SCATT(DIR,TH0)
                  ENDIF
                  THC=ACOS(COSTHC)
                  FIC=2.*PI*RND_NUM()
                  CALL PHOTON_DIR(DIR,THC,FIC,DIRF,POL)
                  DO I=1,3
                     RF0(I)=RF(I)
                     DIRF0(I)=DIRF(I)
                  ENDDO
                  CALL TOP_RAYTRACE(RF,DIRF,POL,E,MDN,TOP,LR,NX,NY,OK)
                  IF(OK) THEN
                     IF(N_PHOT.GE.NSIZ_PHOT) THEN
                        PRINT*,'buffer full: N_PHOT>',NSIZ_PHOT
                        RETURN
                     ENDIF
                     N_PHOT=N_PHOT+1
                     PHOT_M(N_PHOT)=MDN
                     PHOT_LR(N_PHOT)=LR
                     DO I=1,3
                        PHOT_R(I,N_PHOT)=RF(I)
                        PHOT_DIR(I,N_PHOT)=DIRF(I)
                        PHOT_R0(I,N_PHOT)=RF0(I)
                        PHOT_DIR0(I,N_PHOT)=DIRF0(I)
                     ENDDO
                     PHOT_T0(N_PHOT)=T/BETA+MTRA_T0(ITRA)
                     PHOT_T(N_PHOT)=PHOT_T0(N_PHOT)+TOP
                     PHOT_E(N_PHOT)=E
                     PHOT_THC(N_PHOT)=THC
                     PHOT_FIC(N_PHOT)=FIC
                     PHOT_NX(N_PHOT)=NX(1)
                     PHOT_NY(N_PHOT)=NY(1)
                     PHOT_NXM(N_PHOT)=NX(2)
                     PHOT_NYM(N_PHOT)=NY(2)
                     PHOT_NXE(N_PHOT)=NX(3)
                     PHOT_NYE(N_PHOT)=NY(3)
                     PHOT_ICH(N_PHOT)=-1
                     PHOT_MTRA(N_PHOT)=ITRA
                     MTRA_NPH(ITRA)=MTRA_NPH(ITRA)+1
                  ENDIF
               ENDIF
            ENDIF
         ENDIF
      ENDDO

      END



      SUBROUTINE TO_MODULE(RF,DIR,MODULE,OK)
      
      IMPLICIT NONE
      REAL RF(3), DIR(3)
      INTEGER MODULE
      LOGICAL OK
#include "TOP_GEO.fi"
      REAL X, Y, Z
      INTEGER I, K
      LOGICAL INSIDE_QBAR, INSIDE_EXTVOL
      INTEGER MDN /0/
      SAVE MDN

      OK=.FALSE.
      DO K=0,NUM_MOD-1
         I=MOD(K+MDN,NUM_MOD)
         X=RF(1)*QBAR_CFI(I)+RF(2)*QBAR_SFI(I)-QBAR_DX(I)
         IF(ABS(X).GT.QBAR_A(I)/2) CYCLE
         Y=RF(2)*QBAR_CFI(I)-RF(1)*QBAR_SFI(I)-QBAR_R(I)
         Z=RF(3)-QBAR_DZ(I)
         OK=INSIDE_QBAR(X,Y,Z,I).OR.INSIDE_EXTVOL(X,Y,Z,I)
         IF(.NOT.OK) CYCLE
         MODULE=I
         MDN=I
         RF(1)=X
         RF(2)=Y
         RF(3)=Z
         X=DIR(1)*QBAR_CFI(I)+DIR(2)*QBAR_SFI(I)
         Y=DIR(2)*QBAR_CFI(I)-DIR(1)*QBAR_SFI(I)
         DIR(1)=X
         DIR(2)=Y
         RETURN
      ENDDO

      END


      LOGICAL FUNCTION INSIDE_QBAR(XP,YP,ZP,M)

      IMPLICIT NONE
      REAL XP,YP,ZP             ! point
      INTEGER M                 ! module number

#include "TOP_GEO.fi"
      REAL RR, XC, YC, X, Y

      INSIDE_QBAR=.FALSE.
      IF(ABS(XP).GT.QBAR_A(M)/2) RETURN
      IF(ABS(YP).GT.QBAR_B(M)/2) RETURN
      IF(ABS(ZP).GT.QBAR_C(M)/2) RETURN
      IF(QBAR_REDG.GT.0) THEN
         XC=QBAR_A(M)/2-QBAR_REDG
         YC=QBAR_B(M)/2-QBAR_REDG
         X=ABS(XP)-XC
         Y=ABS(YP)-YC
         IF(X.GT.0.AND.Y.GT.0) THEN
            RR=X**2+Y**2
            IF(RR.GT.QBAR_REDG**2) RETURN
         ENDIF
      ENDIF
      IF(ZP.LT.QBAR_ZL(M)) THEN
         IF(QBAR_LSIDE(M).EQ.3) THEN
            RR=(YP-QBAR_YCMI(M))**2+(ZP-QBAR_ZCMI(M))**2
            IF(RR.GT.QBAR_RMI(M)**2) RETURN
         ELSE IF(QBAR_LSIDE(M).EQ.4) THEN
            RR=(XP-QBAR_XCMI(M))**2+(YP-QBAR_YCMI(M))**2+
     &           (ZP-QBAR_ZCMI(M))**2
            IF(RR.GT.QBAR_RMI(M)**2) RETURN
         ENDIF
      ELSE IF(ZP.GT.QBAR_ZR(M)) THEN
         IF(QBAR_RSIDE(M).EQ.3) THEN
            RR=(YP-QBAR_YCMI(M))**2+(ZP-QBAR_ZCMI(M))**2
            IF(RR.GT.QBAR_RMI(M)**2) RETURN
         ELSE IF(QBAR_RSIDE(M).EQ.4) THEN
            RR=(XP-QBAR_XCMI(M))**2+(YP-QBAR_YCMI(M))**2+
     &           (ZP-QBAR_ZCMI(M))**2
            IF(RR.GT.QBAR_RMI(M)**2) RETURN
         ENDIF
      ENDIF

      INSIDE_QBAR=.TRUE.

      END


      LOGICAL FUNCTION INSIDE_EXTVOL(XP,YP,ZP,M)

      IMPLICIT NONE
      REAL XP,YP,ZP             ! point
      INTEGER M                 ! module number

#include "TOP_GEO.fi"
#include "TOP_EXT.fi"
      INTEGER LR, K
      REAL Z, BH, ZZ


      INSIDE_EXTVOL=.FALSE.
      IF(ABS(XP).GT.QBAR_A(M)/2) RETURN
      BH=QBAR_B(M)/2
      ZZ=ZP
      DO LR=0,1
         K=QBAR_EXT(LR,M)
         IF(K.EQ.0) CYCLE
         IF(ZZ.GT.0) CYCLE
         ZZ=-ZZ
         Z=ABS(ZP)-QBAR_C(M)/2
         IF(Z.LT.0.OR.Z.GT.EXT_DZ(K)) CYCLE
         IF(YP.LT.MIN(EXT_YDN(K),-BH)) CYCLE
         IF(YP.GT.MAX(EXT_YUP(K),BH)) CYCLE
         IF(EXT_SHAPE(K).EQ.2) THEN
            IF(YP.GT.(EXT_YUP(K)-BH)/EXT_DZ(K)*Z+BH) CYCLE
            IF(YP.LT.(EXT_YDN(K)+BH)/EXT_DZ(K)*Z-BH) CYCLE
         ENDIF
         INSIDE_EXTVOL=.TRUE.
         RETURN
      ENDDO

      END


      SUBROUTINE TO_GLOBAL(R,DIR,M)

      IMPLICIT NONE
      REAL R(3), DIR(3)
      INTEGER M
#include "TOP_GEO.fi"
      REAL X, Y

      IF(M.LT.0.OR.M.GE.NUM_MOD) RETURN

      X=R(1)+QBAR_DX(M)
      Y=R(2)+QBAR_R(M)
      R(1)=X*QBAR_CFI(M)-Y*QBAR_SFI(M)
      R(2)=Y*QBAR_CFI(M)+X*QBAR_SFI(M)
      R(3)=R(3)+QBAR_DZ(M)
      X=DIR(1)
      Y=DIR(2)
      DIR(1)=X*QBAR_CFI(M)-Y*QBAR_SFI(M)
      DIR(2)=Y*QBAR_CFI(M)+X*QBAR_SFI(M)

      END


      SUBROUTINE TO_LOCAL(R,DIR,M)

      IMPLICIT NONE
      REAL R(3), DIR(3)
      INTEGER M
#include "TOP_GEO.fi"
      REAL X, Y

      IF(M.LT.0.OR.M.GE.NUM_MOD) RETURN

      X=R(1)*QBAR_CFI(M)+R(2)*QBAR_SFI(M)
      Y=R(2)*QBAR_CFI(M)-R(1)*QBAR_SFI(M)
      R(1)=X-QBAR_DX(M)
      R(2)=Y-QBAR_R(M)
      R(3)=R(3)-QBAR_DZ(M)
      X=DIR(1)
      Y=DIR(2)
      DIR(1)=X*QBAR_CFI(M)+Y*QBAR_SFI(M)
      DIR(2)=Y*QBAR_CFI(M)-X*QBAR_SFI(M)

      END



      SUBROUTINE PRESEK_HELIX_TOP(HLX,RHO,T1,T2,OK)

!     crossing of helix and TOP volume

      IMPLICIT NONE
      REAL HLX(10)              ! helix
      REAL RHO                  ! radius particle is
      REAL T1, T2               ! crossing times
      LOGICAL OK                ! T, if helix crosses TOP volume
#include "TOP_GEO.fi"
      REAL T3, T4, Z1, Z2
      LOGICAL OK1

      OK=.FALSE.
      IF(RHO.GE.TOP_R2) RETURN

      CALL XSEC_CYL(HLX,TOP_R1,T1,T2,OK)
      IF(.NOT.OK) RETURN
      IF(RHO.GE.TOP_R1) THEN
         T2=T1
         T1=0.
      ENDIF
      CALL XSEC_CYL(HLX,TOP_R2,T3,T4,OK1)
      IF(OK1) T2=T3

!     z-clip
      IF(HLX(4).NE.0) THEN
         Z1=HLX(5)*T1+HLX(7)
         Z2=HLX(5)*T2+HLX(7)
      ELSE
         Z1=HLX(5)*T1+HLX(1)
         Z2=HLX(5)*T2+HLX(1)
      ENDIF
      OK=MAX(Z1,Z2).GT.TOP_Z1
      IF(.NOT.OK) RETURN
      OK=MIN(Z1,Z2).LT.TOP_Z2
      IF(.NOT.OK) RETURN

      RETURN
      END


      SUBROUTINE MULTIPLE_SCATT(DIR,TH0)

      IMPLICIT NONE
      REAL DIR(3)               ! track direction to be changed [in/out]
      REAL TH0                  ! r.m.s. of scattering angle (th0<<1) [in]
#include "TOP_CONS.fi"
      REAL GAUS_RND
      REAL DIRS(3), S, SMER_Z(3)
      INTEGER I

      IF(TH0.EQ.0) RETURN
      DIRS(1)=GAUS_RND(TH0)
      DIRS(2)=GAUS_RND(TH0)
      DIRS(3)=1.
      S=SQRT(DIRS(1)**2+DIRS(2)**2+DIRS(3)**2)
      DO I=1,3
         DIRS(I)=DIRS(I)/S
      ENDDO
      DO I=1,3
         SMER_Z(I)=DIR(I)
      ENDDO
      CALL ROTATE_TO_FIXED(SMER_Z,DIRS,DIR)

      END


