!-----------------------------------------------------------------------
! TOP counter for Belle: interface to TOP_RTRA
! M. Staric, mar-2009, mar-2010
! units: GeV, GeV/c, cm, ns, cm/ns, T, nm
!-----------------------------------------------------------------------

      SUBROUTINE RTRA_CLEAR

      IMPLICIT NONE
#include "TOP_RTRA.fi"
      DATA N_RTRA /0/
      DATA NHYP /0/

      N_RTRA=0
      END


      SUBROUTINE RTRA_SET_HYPO(NUMHYP,MASSES)

      IMPLICIT NONE
      INTEGER NUMHYP            ! number of mass hypotheses
      REAL MASSES(*)            ! masses
#include "TOP_RTRA.fi"
      INTEGER I

      NHYP=MIN(NUMHYP,NHYP_SIZ)
      DO I=1,NHYP
         RTRA_MASS(I)=MASSES(I)
         RTRA_HYPID(I)=I
      ENDDO

      END


      SUBROUTINE RTRA_SET_HYPID(NUMHYP,HYPID)

      IMPLICIT NONE
      INTEGER NUMHYP            ! number of mass hypotheses
      INTEGER HYPID(*)          ! hypotheses ID (1=e, 2=mu, 3=pi, 4=K, 5=p)
#include "TOP_RTRA.fi"
      INTEGER I

      NHYP=MIN(NUMHYP,NHYP_SIZ)
      DO I=1,NHYP
         RTRA_HYPID(I)=HYPID(I)
      ENDDO

      END


      SUBROUTINE RTRA_GET_HYPID(NUMHYP,HYPID)

      IMPLICIT NONE
      INTEGER NUMHYP            ! number of mass hypotheses
      INTEGER HYPID(*)          ! hypotheses ID (1=e, 2=mu, 3=pi, 4=K, 5=p)
#include "TOP_RTRA.fi"
      INTEGER I

      NHYP=MIN(NUMHYP,NHYP_SIZ)
      DO I=1,NHYP
         HYPID(I)=RTRA_HYPID(I)
      ENDDO

      END


      SUBROUTINE RTRA_PUT(X0,Y0,Z0,T0,PX,PY,PZ,Q,HYP,REF,MCREF)

      IMPLICIT NONE
      REAL X0, Y0, Z0           ! track: point [cm] 
      REAL T0                   ! track: TOF for beta=1 from IP to point [ns]
      REAL PX, PY, PZ           ! track: momentum [GeV/c]
      INTEGER Q                 ! track: charge (+/-1)
      INTEGER HYP               ! hypothesis to measure pulls
      INTEGER REF               ! reference
      INTEGER MCREF             ! reference used in MTRA_PUT (if >0)
#include "TOP_RTRA.fi"
#include "TOP_MTRA.fi"
      INTEGER K, I
      REAL P

      K=N_RTRA+1
      IF(K.GT.NSIZ_RTRA) THEN
         PRINT*,'TOP_RTRA: no space for new track'
         RETURN
      ENDIF
      RTRA_R0(1,K)=X0
      RTRA_R0(2,K)=Y0
      RTRA_R0(3,K)=Z0
      RTRA_T0(K)=T0
      P=SQRT(PX**2+PY**2+PZ**2)
      RTRA_P(K)=P
      IF(P.EQ.0) P=1.
      RTRA_DIR(1,K)=PX/P
      RTRA_DIR(2,K)=PY/P
      RTRA_DIR(3,K)=PZ/P
      RTRA_CHARGE(K)=Q
      DO I=1,NHYP
         RTRA_PLKH(I,K)=0.
         RTRA_SFOT(I,K)=0.
      ENDDO
      RTRA_NFOT(K)=0
      RTRA_FLAG(K)=0
      RTRA_MOD(K)=-1
      DO I=1,3
         RTRA_TR(I,K)=0.
         RTRA_TDIR(I,K)=0.
      ENDDO
      RTRA_TLEN(K)=0.
      RTRA_TOFC0(K)=0.
      RTRA_MTRA(K)=0
      IF(MCREF.GT.0) THEN
         DO I=1,N_MTRA
            IF(MCREF.EQ.MTRA_REF(I)) THEN
               MTRA_RTRA(I)=K
               RTRA_MTRA(K)=I
               EXIT
            ENDIF
         ENDDO
      ENDIF
      RTRA_REF(K)=REF
      RTRA_HYP(K)=HYP
      N_RTRA=K

      END


      INTEGER FUNCTION RTRA_GETNUM()

      IMPLICIT NONE
#include "TOP_RTRA.fi"
      
      RTRA_GETNUM=N_RTRA
      END


      INTEGER FUNCTION RTRA_GETFLAG(K)

      IMPLICIT NONE
      INTEGER K                 ! index (row#) in TOP_RTRA [in]
#include "TOP_RTRA.fi"
      
      IF(K.GT.0.AND.K.LE.N_RTRA) THEN
         RTRA_GETFLAG=RTRA_FLAG(K)
      ELSE
         RTRA_GETFLAG=0
      ENDIF
      END


      SUBROUTINE RTRA_GET(K,PLKH,SFOT,NB,NFOT,FLAG,MTRA,REF)

      IMPLICIT NONE
      INTEGER K                 ! index (row#) in TOP_RTRA [in]
      REAL PLKH(*)              ! log likelihoods
      REAL SFOT(*)              ! expected number of photons
      INTEGER NB                ! size of output arrays
      INTEGER NFOT              ! number of detected photons
      INTEGER FLAG              ! flag
      INTEGER MTRA              ! pointer to MC particle
      INTEGER REF               ! reference
#include "TOP_RTRA.fi"
      INTEGER I

      DO I=1,NB
         PLKH(I)=0.
         SFOT(I)=0.
      ENDDO
      IF(K.GT.0.AND.K.LE.N_RTRA) THEN
         DO I=1,MIN(NHYP,NB)
            PLKH(I)=RTRA_PLKH(I,K)
            SFOT(I)=RTRA_SFOT(I,K)
         ENDDO
         NFOT=RTRA_NFOT(K)
         FLAG=RTRA_FLAG(K)
         MTRA=RTRA_MTRA(K)
         REF=RTRA_REF(K)
      ELSE
         NFOT=0
         FLAG=0
         MTRA=0
         REF=0
      ENDIF
      
      END


      SUBROUTINE RTRA_GETHIT(K,LOC,R,DIR,TLEN,TOF,P,MDN)

      IMPLICIT NONE
      INTEGER K                 ! index (row#) in TOP_RTRA [in]
      INTEGER LOC               ! 0/1 in local/global frame [in]
      REAL R(3)                 ! track point at TOP (local/global) [cm]
      REAL DIR(3)               ! track direction at TOP (local/global)
      REAL TLEN                 ! track length in Q-bar [cm]
      REAL TOF                  ! time-of-flight for beta=1 [ns]
      REAL P                    ! track momentum [GeV/c]
      INTEGER MDN               ! Q-bar ID
#include "TOP_RTRA.fi"
      INTEGER I

      IF(K.GT.0.AND.K.LE.N_RTRA) THEN
         DO I=1,3
            R(I)=RTRA_TR(I,K)
            DIR(I)=RTRA_TDIR(I,K)
         ENDDO
         TLEN=RTRA_TLEN(K)
         TOF=RTRA_T0(K)+RTRA_TOFC0(K)
         P=RTRA_P(K)
         MDN=RTRA_MOD(K)
         IF(LOC.NE.0) CALL TO_GLOBAL(R,DIR,MDN)
      ELSE
         DO I=1,3
            R(I)=0.
            DIR(I)=0.
         ENDDO
         TLEN=0.
         TOF=0.
         P=0.
         MDN=-1
      ENDIF
      
      END


      SUBROUTINE TRACK2TOP(R0,PV,Q,TOF,MDN)

      IMPLICIT NONE
      REAL R0(3)                ! start point [in], at TOP [out]
      REAL PV(3)                ! momentum vector [in/out]
      REAL Q                    ! charge (+/-1) [in]
      REAL TOF                  ! time-of-flight for beta=1 from R0 to TOP [out]
      INTEGER MDN               ! Q-bar ID or -1
#include "TOP_GEO.fi"
#include "TOP_CONS.fi"
      REAL HLX(10), HLX_LOC(10)
      REAL R(3), DIR(3), P, T1, T2, RHO
      INTEGER I, K
      LOGICAL OK

      MDN=-1
      P=SQRT(PV(1)**2+PV(2)**2+PV(3)**2)
      IF(P.LT.1E-4) RETURN
      DO I=1,3
         DIR(I)=PV(I)/P
      ENDDO
      CALL SET_HELIX(R0,DIR,P,Q,HLX)
      RHO=SQRT(R0(1)**2+R0(2)**2)
      TOF=0
      IF(RHO.NE.TOP_R1) THEN
         CALL XSEC_CYL(HLX,TOP_R1,T1,T2,OK)
         IF(.NOT.OK) RETURN
         IF(RHO.LT.TOP_R1) THEN
            TOF=T1
         ELSE
            TOF=T2-HLX(10)
         ENDIF
         CALL GET_HLX_RDIR(HLX,TOF,R0,DIR)
         DO I=1,3
            PV(I)=DIR(I)*P
         ENDDO
      ENDIF

      DO K=0,NUM_MOD-1
         CALL HLX_TO_LOCAL(HLX,K,HLX_LOC)
         CALL XSEC_PLANE_Y(HLX_LOC,0.,T1,T2,OK)
         IF(.NOT.OK) CYCLE
         CALL GET_HLX_RDIR(HLX_LOC,T1,R,DIR)
         OK=ABS(R(1)).LT.QBAR_A(K)/2.
         IF(.NOT.OK) CYCLE
         OK=ABS(R(3)).LT.QBAR_C(K)/2.
         IF(.NOT.OK) CYCLE
         MDN=K
         TOF=T1
         CALL GET_HLX_RDIR(HLX,T1,R0,DIR)
         DO I=1,3
            PV(I)=DIR(I)*P
         ENDDO
         RETURN
      ENDDO

      END

